# all imports
from flask import Flask, jsonify, request
from flask_socketio import SocketIO
from flask_cors import CORS

# Code voor led
from helpers.klasseknop import Button
from RPi import GPIO
import time
from subprocess import check_output
from datetime import datetime
from helpers.Database import Database
import threading

from serial import Serial, PARITY_NONE
from pwm_led import PWM_led

# setup
count = 0
stringline = ''
weer_gevolgd = False

roodled = PWM_led(27)
groenled = PWM_led(22)
blauwled = PWM_led(5)
witled = PWM_led(17)

roodled.start_frequency(1)
groenled.start_frequency(1)
blauwled.start_frequency(1)
witled.start_frequency(1)



GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)


app = Flask(__name__)
CORS(app)
socketio = SocketIO(app)

conn = Database(app=app, user='mct', password='mct',
                db='databaseV2', host='localhost', port=3306)

# connectie socket.io
# def connect():
#     @app.route('/')
#     def hallo():
#         return "Server is running"


@socketio.on("connect")
def connecting():
    socketio.emit("connected")
    print("Connection with client established")

    tijd = str(datetime.now())
    nu = tijd[10:16]
    print(nu)
    socketio.emit('nu', nu)
    data_halen_temperatuur = conn.get_data("SELECT raspberryPi.waarde FROM raspberryPi WHERE sensors_sensorsID = %s", 2)
    js_tempdata = str(data_halen_temperatuur)
    socketio.emit('temperatuur_js', js_tempdata)

@socketio.on("rood")
def rood():
    print('de kleur is rood')
    roodled.start_pwm(100)
    blauwled.change_frequency(1)
    witled.change_frequency(1)
    groenled.change_frequency(1)

@socketio.on("groen")
def groen():
    print('de kleur is groen')
    roodled.change_frequency(1)
    blauwled.change_frequency(1)
    witled.change_frequency(1)
    groenled.start_pwm(100)

@socketio.on("blauw")
def blauw():
    print('de kleur is blauw')
    roodled.stop_pwm()
    blauwled.start_pwm(100)
    witled.stop_pwm()
    groenled.stop_pwm()

@socketio.on("wit")
def wit():
    print('de kleur is wit')
    roodled.stop_pwm()
    blauwled.stop_pwm()
    witled.start_pwm(100)
    groenled.stop_pwm()


@socketio.on("weer_volgen")
def weer_volgen():
    global count
    if count == 0:
        print('change aan')
        count = 1
        print(count)
    elif count == 1:
        print('change uit')
        count = 0
        print(count)


# lcd setup
ips = check_output(['hostname', '--all-ip-addresses'])
ips_kort = ips[:12]


print("script is running")

D0 = 16
D1 = 12
D2 = 25
D3 = 24
D4 = 23
D5 = 26
D6 = 19
D7 = 13
RS = 21
E = 20
databus = [D7, D6, D5, D4, D3, D2, D1, D0]
alfabet = {
    ' ': 0x20,
    "'": 0x27,
    '.': 0x2E,
    '/': 0x2F,
    '0': 0x30,
    '1': 0x31,
    '2': 0x32,
    '3': 0x33,
    '4': 0x34,
    '5': 0x35,
    '6': 0x36,
    '7': 0x37,
    '8': 0x38,
    '9': 0x39,
    ':': 0x3A,
    ';': 0x3B,
    '<': 0x3C,
    '=': 0x3D,
    '>': 0x3E,
    '?': 0x3F,
    '@': 0x40,
    'A': 0x41,
    'B': 0x42,
    'C': 0x43,
    'D': 0x44,
    'E': 0x45,
    'F': 0x46,
    'G': 0x47,
    'H': 0x48,
    'I': 0x49,
    'J': 0x4A,
    'K': 0x4B,
    'L': 0x4C,
    'M': 0x4D,
    'N': 0x4E,
    'O': 0x4F,
    'P': 0x50,
    'Q': 0x51,
    'R': 0x52,
    'S': 0x53,
    'T': 0x54,
    'U': 0x55,
    'V': 0x56,
    'W': 0x57,
    'X': 0x58,
    'Y': 0x59,
    'Z': 0x5A
}

wait = 0.01

def setup():
    GPIO.setmode(GPIO.BCM)
    for initial in databus:
        GPIO.setup(initial, GPIO.OUT)
    GPIO.setup(RS, GPIO.OUT)
    GPIO.setup(E, GPIO.OUT)


def send_instruction(value):
    GPIO.output(RS, GPIO.LOW)
    GPIO.output(E, GPIO.HIGH)

    set_data_bits(value)
    GPIO.output(E, GPIO.LOW)


def init_lcd():
    send_instruction(0x38)
    send_instruction(0x0F)
    send_instruction(0x01)


def send_character(value):
    GPIO.output(RS, GPIO.HIGH)
    GPIO.output(E, GPIO.HIGH)

    set_data_bits(value)
    GPIO.output(E, GPIO.LOW)


def set_data_bits(value):
    mask = 0x80
    for i in range(0, 8):
        if mask & value:
            GPIO.output(databus[i], GPIO.HIGH)
        else:
            GPIO.output(databus[i], GPIO.LOW)

        mask = mask >> 1
        time.sleep(0.01)


def write_message(message):
    for i in message:
        send_character(alfabet.get(i.upper()))


def second_row():
        send_instruction(0xC0)

# @socketio.on('nu')
# def handle_tijd():
#     tijd = str(datetime.datetime.now())
#     nu = tijd[10:16]
#     print(nu)
#     socketio.emit('nu', nu)


#     gedrukt = lees_knop(20)
#     socketio.emit("op de knop gedrukt (flask)", gedrukt)
#     print("gedrukt op de site")
#     global count
#     js = request.get_json()
#     if count == 0:
#         print('knop aan')
#         count = 1
#         data_aan = conn.set_data("INSERT INTO data_van_knop (knop, tijd) VALUES('aan', current_timestamp)")
#         return jsonify(data_aan), 201
#     elif count == 1:
#         print('knop uit')
#         count = 0
#         data_uit = conn.set_data("INSERT INTO data_van_knop (knop, tijd) VALUES('uit', current_timestamp)")
#         return jsonify(data_uit), 201

# seriele connectie met arduino
def init():
    with Serial('/dev/serial0', 9600, bytesize=8, parity=PARITY_NONE, stopbits=1) as port:
        while True:
            port.write(b"communication ")
            line = port.readline()
            stringline = line.decode(encoding='utf-8')
            # print("van arduino terug : " + stringline)
            # count+= 1
            # print(count)
            # js = request.get_json()
            if stringline[:1] == "A":
                temp = stringline[1:5]
                int_temp = float(temp)
                print(int_temp)
                socketio.emit('temperatuur_js', int_temp)
                print("temperatuur van arduino: " + temp)
                data_temperatuur = conn.set_data("INSERT INTO raspberryPi (dateTime, sensors_sensorsID, actuators_actuatorsID, waarde, actie) "
                                                 "VALUES(current_timestamp, %s, %s, %s, %s)", [[2], [1], [int_temp], ['temperatuur invoegen']])
                if int_temp < 18:
                    print('koud')
                else:
                    print('warm')
            elif stringline[:1] == "B":
                vocht = stringline[1:]
                int_vocht = float(vocht)
                print("vochtigheid van arduino: " + vocht)
                data_vochtigheid = conn.set_data(
                    "INSERT INTO raspberryPi (dateTime, sensors_sensorsID, actuators_actuatorsID, waarde, actie) "
                    "VALUES(current_timestamp, %s, %s, %s, %s)", [[3], [1], [int_vocht], ['vochtigheid invoegen']])
                if int_vocht < 50:
                    print('droog')
                elif int_vocht > 50 and int_vocht < 70:
                    print('vochtig')
                else:
                    print('nat')
                    if count==1:
                        blauwled.start_frequency(2000)
                        while True:
                            roodled.stop_pwm()
                            blauwled.start_pwm(100)
                            witled.stop_pwm()
                            groenled.stop_pwm()
            elif stringline[:1] == "C":
                lux = stringline[1:]
                int_lux = float(lux)
                print("lux van arduino: " + lux)
                data_lux = conn.set_data(
                    "INSERT INTO raspberryPi (dateTime, sensors_sensorsID, actuators_actuatorsID, waarde, actie) "
                    "VALUES(current_timestamp, %s, %s, %s, %s)", [[4], [1], [int_lux], ['lux waarde invoegen']])
                if int_lux < 190:
                    print('nacht')
                else:
                    print('dag')
            else:
                print('error')
            time.sleep(20)

def leds():
    rood.start_pwm(100)
    groen.start_pwm(100)
    blauw.start_pwm(100)
    wit.start_pwm(26)

# lcd aansturen
def lcd():
    try:
        while True:
            setup()
            init_lcd()
            datumtijd = str(datetime.now())
            time_kort = datumtijd[11:16]
            write_message(str(time_kort))
            print(ips)
            print(ips_kort)
            second_row()
            write_message(str(ips_kort))
            print(time_kort)
            # print(alfabet)
            time.sleep(60)
    except KeyboardInterrupt as e:
        print(e)
    finally:
        GPIO.cleanup()
        print("Script has stopped")



# threading definieren
thread_init = threading.Thread(target=init)
thread_led = threading.Thread(target=leds)
thread_lcd = threading.Thread(target=lcd)
#thread_socketio = threading.Thread(target=connect)


# start code
#thread_socketio.start()
thread_init.start()
thread_lcd.start()
#thread_led.start()

# verbind met frondend
if __name__ == '__main__':
    socketio.run(app, host="0.0.0.0", port=5000)






# def lees_knop(pin):
#     if GPIO.input(led1) == 1:
#         GPIO.output(led1, GPIO.LOW)
#     else:
#         GPIO.output(led1, GPIO.HIGH)
#     print("button pressed")
